package device.ui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import device.MoneyMechanismEvents;
import device.ConsoleMechanismEvents;
import device.OutMechanismEvents;

public class InterfaceBuilder {
	public final static float TOP_MARGIN = 0.07f;

	public final static float LEFT_MARGIN = 0.04f;

	public final static float RIGHT_MARGIN = 0.04f;

	public final static float SCREEN_SIZE = 0.45f;

	public final static float FUNC_MARGIN = 0.02f;

	public final static float FUNC_VSIZE = 0.1f;

	public final static float FUNC_HSIZE = 0.05f;

	public final static float MONEY_MARGIN = 0.04f;

	public final static float MONEY_HSIZE = 0.35f;

	public final static float MONEY_VSIZE = 0.04f;

	public final static float MONEY_INSIDE_VSIZE = 0.8f;

	public final static float MONEY_INSIDE_HSIZE = 0.2f;

	public final static float OUTPUT_MARGIN = 0.02f;

	public final static float OUTPUT_VSIZE = 0.14f;

	public final static int LAMP_HCOUNT = 5;

	public final static int LAMP_VCOUNT = 3;

	public final static int LAMP_SIZE = 5;

	public final static int IMAGE_MARGIN = 20;

	public final static int WIDTH = 420;

	public final static int HEIGHT = 575;

	public final static int EDGE_COLOR = 0;

	public final static int SCREEN_COLOR = 0xCCCCEE;

	public final static int MONEY_ALLOWED = 3329330;

	public final static int MONEY_DISALLOWED = 0xFF0000;

	public int screen = 1;

	public static int chosenTicket = 0;

	public String[] ticketTypesRu = { "  ", " ",
			" ", " " };

	public String[] ticketTypesEng = { "Month unlimited ride", "Ten pass",
			"Five pass", "Single pass" };

	public static int[] ticketPrice = { 200, 50, 25, 5 };

	public boolean[] moneyAvailable = { true, true, true, true, true, true };

	volatile boolean interfaceLoaded = false;

	MyButton[] assignations;

	MyButton[] pickableAssignations;

	public int[] assignationNominals = { 100, 50, 10, 5, 2, 1 };

	InterfaceState interfaceState = new InterfaceState();

	String backString = "uniground.png";

	public MyButton btnRu = new MyButton(), btnEng = new MyButton(),
			btnLogo = new MyButton(), btnBack = new MyButton(),
			btnPay = new MyButton(), btnChange = new MyButton(),
			btnTicket = new MyButton();

	MyLabeledButton btnCh[];

	MyRegion outputDevice = null;

	MainForm mainForm;

	void loadInterface(MainForm _mainForm) {
		mainForm = _mainForm;
		btnRu.load("rus_flag.png");
		btnRu.setActive();
		btnEng.load("uk_flag.png");
		btnLogo.load("metro.png");
		btnBack.load("back.png");

		btnCh = new MyLabeledButton[4];
		for (int i = 0; i < 4; i++) {
			btnCh[i] = new MyLabeledButton();
			if (i == 0) {
				btnCh[i].load("button.png");
			} else {
				btnCh[i].setTheSameImage(btnCh[0]);
			}
			btnCh[i].textRu = ticketTypesRu[i] + ", " + ticketPrice[i] + ".";
			btnCh[i].textEng = ticketTypesEng[i] + ", " + ticketPrice[i]
					+ "rub.";
		}

		btnPay.load("pay_NA.png");
		btnChange.load("change.png");
		btnTicket.load("smart_card.png");

		assignations = new MyButton[6];
		for (int i = 0; i < 6; i++) {
			assignations[i] = new MyButton();
		}
		assignations[0].load("rub100.png");
		assignations[1].load("rub50.png");
		assignations[2].load("rub10.png");
		assignations[3].load("rub5.png");
		assignations[4].load("rub2.png");
		assignations[5].load("rub1.png");

		pickableAssignations = new MyButton[6];
		for (int i = 0; i < 6; i++) {
			pickableAssignations[i] = new MyButton();
		}

		pickableAssignations[0].load("rub100.png");
		pickableAssignations[1].load("rub50.png");
		pickableAssignations[2].load("rub10.png");
		pickableAssignations[3].load("rub5.png");
		pickableAssignations[4].load("rub2.png");
		pickableAssignations[5].load("rub1.png");

		mainForm.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent ev) {
				int x = ev.getX();
				int y = ev.getY();
				Rectangle boundingRect = mainForm.getBounds();
				int boundingWidth, boundingHeight;
				boundingWidth = boundingRect.width;
				boundingHeight = boundingRect.height;
				if (btnRu.test(x, y) && !InterfaceState.language) {
					InterfaceState.switchLanguage();
					btnRu.setActive();
					btnEng.setPassive();
					for (int i = 0; i < 4; i++) {
						btnCh[i].switchLanguage();
					}
					mainForm.repaint();
				}
				if (btnEng.test(x, y) && InterfaceState.language) {
					InterfaceState.switchLanguage();
					btnEng.setActive();
					btnRu.setPassive();
					for (int i = 0; i < 4; i++) {
						btnCh[i].switchLanguage();
					}
					mainForm.repaint();
				}
				if (screen == 1) {
					for (int i = 0; i < 4; i++) {
						if (btnCh[i].test(x, y)) {
							chosenTicket = i;
							mainForm.notify(ConsoleMechanismEvents.E1);
							mainForm.repaint();
						}
					}
				}
				if (screen > 1 && btnBack.test(x, y)) {
					btnPay.load("pay_NA.png");
					mainForm.notify(ConsoleMechanismEvents.E4);
					mainForm.repaint();
				}
				if (screen == 2) {
					for (int i = 5; i >= 0; i--) {
						if (pickableAssignations[i].test(x, y)
								&& interfaceState.moneyEnabled == true) {
							InterfaceState.coinsIn += assignationNominals[i];
							mainForm.notify(MoneyMechanismEvents.E2);
							mainForm.repaint();
							break;
						}
					}
					if (btnPay.test(x, y) && btnPay.fileName == "pay.png") {
						mainForm.notify(ConsoleMechanismEvents.E3);
						mainForm.repaint();
					}
				}
				if (outputDevice != null && outputDevice.test(x, y)) {
					if (screen == 3) {
						mainForm.notify(OutMechanismEvents.E5);
					}
					if (screen == 4) {
						mainForm.notify(OutMechanismEvents.E7);
					}
				}
			}
		});
		interfaceLoaded = true;
	}

	void render(Graphics g) {
		Rectangle boundingRect = mainForm.getBounds();
		while (!interfaceLoaded)
			;
		int boundingWidth, boundingHeight;
		boundingWidth = boundingRect.width;
		boundingHeight = boundingRect.height;
		// Refill screen
		g.setColor(new Color(0x3f3fa9));
		g.fillRect(0, 0, WIDTH, HEIGHT);
		// Screen
		MyRegion screenRegion = new MyRegion(g, Math.round(boundingWidth
				* LEFT_MARGIN), Math.round(boundingHeight * TOP_MARGIN), Math
				.round(boundingHeight * SCREEN_SIZE), Math.round(boundingWidth
				* (1 - RIGHT_MARGIN - LEFT_MARGIN)));
		screenRegion.setBackColor(new Color(SCREEN_COLOR));
		screenRegion.setFrameColor(new Color(EDGE_COLOR));
		screenRegion.render();
		// Inside the screen
		// RUS & ENG language choice
		while (!btnRu.loaded || !btnEng.loaded)
			;
		for (int i = 0; i < 6; i++) {
			while (!assignations[0].loaded)
				;
			while (!pickableAssignations[0].loaded)
				;
		}
		btnRu.applyLeftTopPosition(screenRegion.getShifted(IMAGE_MARGIN,
				IMAGE_MARGIN));
		btnRu.draw(g);
		btnEng.applyLeftTopPosition(btnRu.getRegion().getShifted(
				IMAGE_MARGIN + btnRu.getRegion().getWidth(), 0));
		btnEng.draw(g);
		if (screen <= 4) {
			btnBack.applyLeftTopPosition(btnEng.getRegion().getShifted(
					IMAGE_MARGIN + btnEng.getRegion().getWidth(), 0));
		}
		if (screen == 2) {
			btnBack.draw(g);
		}
		btnLogo.applyLeftTopPosition(screenRegion.getShifted(screenRegion
				.getWidth()
				- 10 - btnLogo.getRegion().getWidth(), 10));
		btnLogo.draw(g);
		if (screen == 1) {
			btnCh[0].applyLeftTopPosition(screenRegion.getShifted(
					IMAGE_MARGIN + 10, Math.round(2.5f * btnEng.getRegion()
							.getHeight())));
			btnCh[0].draw(g);
			for (int i = 1; i < 4; i++) {
				btnCh[i].applyLeftTopPosition(btnCh[i - 1].getRegion()
						.getShifted(
								0,
								Math.round(1.5f * btnCh[0].getRegion()
										.getHeight())));
				btnCh[i].draw(g);
			}
		}
		if (screen == 2) {
			MyRegion ticketTypeLabel = screenRegion.getShifted(
					IMAGE_MARGIN + 20, 3 * btnEng.getRegion().getHeight() + 10);
			Font font = new Font("Tahoma", Font.PLAIN, 22);
			g.setFont(font);
			if (InterfaceState.language) {
				ticketTypeLabel.renderString(" : "
						+ ticketTypesRu[chosenTicket]);
			} else {
				ticketTypeLabel.renderString("Ticket type: "
						+ ticketTypesEng[chosenTicket]);
			}
			MyRegion priceLabel = ticketTypeLabel.getShifted(0, 10 + btnEng
					.getRegion().getHeight());
			if (InterfaceState.language) {
				priceLabel.renderString(" : "
						+ ticketPrice[chosenTicket] + ".");
			} else {
				priceLabel.renderString("Ticket price: "
						+ ticketPrice[chosenTicket] + "rub.");
			}
			MyRegion assignationsLabel = screenRegion
					.getShifted(IMAGE_MARGIN, -Math
							.round(1.7f * assignations[0].getRegion()
									.getHeight()));
			if (InterfaceState.language) {
				assignationsLabel.renderString("   :");
			} else {
				assignationsLabel.renderString("Notes and coins available:");
			}
			assignations[0].applyLeftTopPosition(screenRegion.getShifted(
					IMAGE_MARGIN, -Math.round(1.5f * assignations[0]
							.getRegion().getHeight()), false, true));
			assignations[0].draw(g);
			for (int i = 1; i < 5; i++) {
				assignations[i].applyLeftTopPosition(assignations[i - 1]
						.getRegion().getShifted(
								Math.round(assignations[i - 1].getRegion()
										.getWidth() * 0.2f), 0));
				assignations[i].draw(g);
			}
			for (int i = 5; i < 6; i++) {
				assignations[i].applyLeftTopPosition(assignations[i - 1]
						.getRegion().getShifted(
								Math.round(assignations[i - 1].getRegion()
										.getWidth() * 0.6f), 0));
				assignations[i].draw(g);
			}
			Font newFont = new Font("Tahoma", Font.BOLD, 36);
			g.setFont(newFont);
			String label = Integer.toString(InterfaceState.coinsIn);
			if (InterfaceState.language) {
				label += ".";
			} else {
				label += "r.";
			}
			MyRegion coinsLabel = screenRegion.getShifted(-22 * label.length(),
					-80, true, true);
			coinsLabel.renderString(label);
			btnPay.applyLeftTopPosition(screenRegion.getShifted(-80, -60, true,
					true));
			btnPay.draw(g);
		}
		if (screen == 3) {
			Font newFont = new Font("Tahoma", Font.BOLD, 36);
			g.setFont(newFont);
			String label;
			if (InterfaceState.language) {
				label = " ";
			} else {
				label = "Take a ticket";
			}
			MyRegion takeTicketLabel = screenRegion.getShifted(0, screenRegion
					.getHeight() / 2 - 20);
			if (!(InterfaceState.coinsIn > ticketPrice[chosenTicket])) {
				takeTicketLabel = takeTicketLabel.getShifted(0, 45);
			}
			takeTicketLabel.renderString(label, screenRegion);
			if (InterfaceState.coinsIn > ticketPrice[chosenTicket]) {
				takeTicketLabel = takeTicketLabel.getShifted(0, 45);
				if (InterfaceState.language) {
					label = "";
				} else {
					label = "and";
				}
				takeTicketLabel.renderString(label, screenRegion);
				takeTicketLabel = takeTicketLabel.getShifted(0, 45);
				if (InterfaceState.language) {
					label = "";
				} else {
					label = "change";
				}
				takeTicketLabel.renderString(label, screenRegion);
			}
		}
		if (screen == 4) {
			Font newFont = new Font("Tahoma", Font.BOLD, 36);
			g.setFont(newFont);
			String label;
			if (InterfaceState.language) {
				label = " ";
			} else {
				label = "Take money back";
			}
			MyRegion moneyBackLabel = screenRegion.getShifted(0, screenRegion
					.getHeight() / 2 + 25);
			moneyBackLabel.renderString(label, screenRegion);
		}
		if (screen == 5) {
			Font newFont = new Font("Tahoma", Font.BOLD, 36);
			g.setFont(newFont);
			String label;
			if (InterfaceState.language) {
				label = "";
			} else {
				label = "Device";
			}
			MyRegion deviceOutOfService = screenRegion.getShifted(0,
					screenRegion.getHeight() / 2 - 10);
			deviceOutOfService.renderString(label, screenRegion);
			if (InterfaceState.language) {
				label = " !";
			} else {
				label = "doesn't work!";
			}
			deviceOutOfService = deviceOutOfService.getShifted(0, 45);
			deviceOutOfService.renderString(label, screenRegion);
		}
		// Coin outside
		MyRegion coinOutside = screenRegion.getShifted(0, Math
				.round(boundingHeight * FUNC_MARGIN), false, true);
		coinOutside.setDimensions(Math.round(boundingWidth * FUNC_HSIZE), Math
				.round(boundingHeight * FUNC_VSIZE));
		if (InterfaceState.moneyEnabled) {
			coinOutside.setBackColor(new Color(MONEY_ALLOWED));
		} else {
			coinOutside.setBackColor(new Color(MONEY_DISALLOWED));
		}
		coinOutside.render();
		// Coin inside
		MyRegion coinInside = coinOutside
				.getShifted(3 * coinOutside.getWidth() / 8 + 1, coinOutside
						.getHeight() / 8);
		coinInside.setDimensions(coinOutside.getWidth() / 4, 3 * coinOutside
				.getHeight() / 4);
		coinInside.setBackColor(new Color(0));
		coinInside.render();
		// Money outside
		MyRegion moneyOutside = coinOutside.getShifted(Math.round(boundingWidth
				* MONEY_MARGIN), 0, true, false);
		moneyOutside.setDimensions(Math.round(boundingWidth * MONEY_HSIZE),
				Math.round(boundingHeight * MONEY_VSIZE));
		if (InterfaceState.moneyEnabled) {
			moneyOutside.setBackColor(new Color(MONEY_ALLOWED));
		} else {
			moneyOutside.setBackColor(new Color(MONEY_DISALLOWED));
		}
		moneyOutside.render();
		// Money inside
		MyRegion moneyInside = moneyOutside.getShifted(Math.round(moneyOutside
				.getWidth()
				* (1 - MONEY_INSIDE_VSIZE) / 2), Math.round(moneyOutside
				.getHeight()
				* (1 - MONEY_INSIDE_HSIZE) / 2));
		moneyInside.setBackColor(new Color(0));
		moneyInside.setDimensions(Math.round(moneyOutside.getWidth()
				* MONEY_INSIDE_VSIZE), Math.round(moneyOutside.getHeight()
				* MONEY_INSIDE_HSIZE));
		moneyInside.render();
		// Output
		outputDevice = coinOutside.getShifted(0, Math.round(boundingHeight
				* OUTPUT_MARGIN), false, true);
		outputDevice.setDimensions(moneyOutside.getWidth()
				+ moneyOutside.getLeft() - coinOutside.getLeft(), Math
				.round(boundingHeight * OUTPUT_VSIZE));
		outputDevice.setBackColor(new Color(0x6666cc));
		outputDevice.render();
		if (screen == 3 && (InterfaceState.coinsIn > ticketPrice[chosenTicket])
				|| screen == 4) {
			outputDevice.setBackColor(new Color(0xcccc66));
			outputDevice.render();
			btnChange.applyLeftTopPosition(outputDevice.getShifted(10, 5));
			btnChange.draw(g);
		}
		if (screen == 3) {
			btnTicket.applyLeftTopPosition(outputDevice.getShifted(75, 10));
			btnTicket.draw(g);
		}
		// Auxilary
		MyRegion auxScreen = moneyOutside.getShifted(20, 0, true, false);
		auxScreen.setWidth(Math.round((1 - RIGHT_MARGIN) * boundingWidth)
				- auxScreen.getLeft());
		auxScreen.setHeight(outputDevice.getTop() + outputDevice.getHeight()
				- auxScreen.getTop());
		auxScreen.setBackColor(new Color(0x353795));
		auxScreen.render();
		// Pickable money
		pickableAssignations[0].getRegion().setLeft(
				Math.round(boundingWidth * LEFT_MARGIN));
		pickableAssignations[0].getRegion().setTop(
				boundingHeight
						- Math.round(1.3f * pickableAssignations[0].getRegion()
								.getHeight()));
		// Divizor
		MyRegion divizor = new MyRegion(g, 0, (outputDevice.getTop()
				+ outputDevice.getHeight() + pickableAssignations[0]
				.getRegion().getTop()) / 2, 1, boundingWidth);
		divizor.setBackColor(new Color(0));
		// Background refill
		MyRegion bottomRefill = divizor.getShifted(0, 0);
		bottomRefill.setBackColor(new Color(0xebe9ed));
		bottomRefill.setHeight(boundingHeight - bottomRefill.getTop());
		bottomRefill.render();
		// Divizor rendering
		divizor.render();
		// <-- Pickable money
		pickableAssignations[0].draw(g);
		for (int i = 1; i < 5; i++) {
			pickableAssignations[i]
					.applyLeftTopPosition(pickableAssignations[i - 1]
							.getRegion().getShifted(
									Math.round(pickableAssignations[i - 1]
											.getRegion().getWidth() * 0.5f), 0));
			pickableAssignations[i].draw(g);
		}
		for (int i = 5; i < 6; i++) {
			pickableAssignations[i]
					.applyLeftTopPosition(pickableAssignations[i - 1]
							.getRegion().getShifted(
									Math.round(pickableAssignations[i - 1]
											.getRegion().getWidth() * 0.8f), 0));
			pickableAssignations[i].draw(g);
		}
	}
}
