package device.ui;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;

public class MyRegion {
	private int left, top, height, width;

	private Graphics g;

	private Color backColor = null, frameColor = null;

	public MyRegion() {
	}

	public MyRegion(Graphics g, int left, int top, int height, int width) {
		this.g = g;
		this.left = left;
		this.top = top;
		this.height = height;
		this.width = width;
	}

	public MyRegion getShifted(int rightShift, int downShift) {
		MyRegion newRegion = new MyRegion(g, left + rightShift,
				top + downShift, height, width);
		return newRegion;
	}

	public MyRegion getShifted(int rightShift, int downShift,
			boolean fromRight, boolean fromBottom) {
		if (fromRight) {
			rightShift += width;
		}
		if (fromBottom) {
			downShift += height;
		}
		MyRegion newRegion = new MyRegion(g, left + rightShift,
				top + downShift, height, width);
		return newRegion;
	}

	boolean test(int x, int y) {
		return (x >= left && x <= left + width && y >= top && y <= top + height);
	}

	public void render() {

		Color tmpColor = g.getColor();
		if (backColor != null) {
			g.setColor(backColor);
			g.fillRect(left, top, width, height);
		}
		if (frameColor != null) {
			g.setColor(frameColor);
		} else {
			g.setColor(new Color(0));
		}
		g.drawRect(left, top, width, height);
		g.setColor(tmpColor);
	}

	public void renderString(String text) {
		g.drawString(text, left, top);
	}

	public void renderString(String text, MyRegion centerBy) {
		FontMetrics fontMetrics = g.getFontMetrics();
		left = centerBy.left
				+ (centerBy.getWidth() - fontMetrics.stringWidth(text)) / 2;
		g.drawString(text, left, top);
	}

	public void setBackColor(Color backColor) {
		this.backColor = backColor;
	}

	public void setFrameColor(Color frameColor) {
		this.frameColor = frameColor;
	}

	public int getHeight() {
		return height;
	}

	public int getLeft() {
		return left;
	}

	public int getTop() {
		return top;
	}

	public int getWidth() {
		return width;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setLeft(int left) {
		this.left = left;
	}

	public void setTop(int top) {
		this.top = top;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public void setDimensions(int width, int height) {
		this.width = width;
		this.height = height;
	}
}
